﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.UserGroupManager.Common;
using Benday.UserGroupManager.Presentation;

namespace Benday.UserGroupManager.UnitTests.Presentation
{
    [TestClass]
    public class MemberEditorViewModelFixture : SilverlightUnitTestBase
    {
        public MemberEditorViewModelFixture()
        {
        }

        [TestMethod]
        public void MemberEditorViewModel_Constructor_PopulatesValuesCorrectlyFromModelInstance()
        {
            IMember model = MockRepositoryInstance.Stub<IMember>();

            int expectedId = 1234;
            string expectedFirstName = "first name";
            string expectedLastName = "last name";
            string expectedEmailAddress = "email address";
            bool expectedReceiveEmails = true;
            
            model.Id = expectedId;
            model.FirstName = expectedFirstName;
            model.LastName = expectedLastName;
            model.EmailAddress = expectedEmailAddress;            
            model.ReceiveEmails = expectedReceiveEmails;

            var viewModel = new MemberEditorViewModel(model, MockRepositoryInstance.Stub<IMemberService>());

            Assert.AreEqual<int>(expectedId, viewModel.Id, "Id did not match.");
            Assert.AreEqual<string>(expectedFirstName, viewModel.FirstName, "FirstName did not match.");
            Assert.AreEqual<string>(expectedLastName, viewModel.LastName, "LastName did not match.");
            Assert.AreEqual<string>(expectedEmailAddress, viewModel.EmailAddress, "EmailAddress did not match.");
            Assert.AreEqual<bool>(expectedReceiveEmails, viewModel.ReceiveEmails, "ReceiveEmails did not match.");
        }

        [TestMethod]
        public void MemberEditorViewModel_Constructor_EmptyCtorInitializesValues()
        {
            var viewModel = new MemberEditorViewModel(MockRepositoryInstance.Stub<IMemberService>());

            Assert.AreEqual<int>(0, viewModel.Id, "Id");
            Assert.AreEqual<string>(String.Empty, viewModel.FirstName, "FirstName");
            Assert.AreEqual<string>(String.Empty, viewModel.LastName, "LastName");
            Assert.AreEqual<string>(String.Empty, viewModel.EmailAddress, "EmailAddress");
            Assert.AreEqual<bool>(false, viewModel.ReceiveEmails, "ReceiveEmails");
        }

        [TestMethod]
        public void MemberEditorViewModel_NotifyPropertyChanged_GetEventsWhenValuesChange()
        {
            var viewModel = new MemberEditorViewModel(MockRepositoryInstance.Stub<IMemberService>());

            Assert.AreEqual<int>(0, viewModel.Id, "Id");
            Assert.AreEqual<string>(String.Empty, viewModel.FirstName, "FirstName");
            Assert.AreEqual<string>(String.Empty, viewModel.LastName, "LastName");
            Assert.AreEqual<string>(String.Empty, viewModel.EmailAddress, "EmailAddress");
            Assert.AreEqual<bool>(false, viewModel.ReceiveEmails, "ReceiveEmails");

            NotifyPropertyChangedTester tester = new NotifyPropertyChangedTester(viewModel);

            Assert.AreEqual<int>(0, tester.Changes.Count, "Changes count was wrong.");

            viewModel.Id = 1234;
            viewModel.FirstName = "fn";
            viewModel.LastName = "ln";
            viewModel.ReceiveEmails = true;
            viewModel.EmailAddress = "email";

            Assert.AreEqual<int>(5, tester.Changes.Count, "Changes count was wrong.");

            tester.AssertChange(0, "Id");
            tester.AssertChange(1, "FirstName");
            tester.AssertChange(2, "LastName");
            tester.AssertChange(3, "ReceiveEmails");
            tester.AssertChange(4, "EmailAddress");
        }

        [TestMethod]
        public void MemberEditorViewModel_Save_PopulatesTheModel()
        {
            IMember model = MockRepositoryInstance.Stub<IMember>();

            model.Id = 1234;
            model.FirstName = "first name";
            model.LastName = "last name";
            model.EmailAddress = "email address";
            model.ReceiveEmails = true;

            var viewModel = new MemberEditorViewModel(model, MockRepositoryInstance.Stub<IMemberService>());

            viewModel.Id = 23456;
            viewModel.FirstName = "updated fn";
            viewModel.LastName = "updated ln";
            viewModel.EmailAddress = "updated email";
            viewModel.ReceiveEmails = !(viewModel.ReceiveEmails);

            viewModel.Save();

            AssertAreEqual(viewModel, model);
        }

        [TestMethod]
        public void MemberEditorViewModel_Save_CallsMemberServiceSave()
        {
            IMember model = MockRepositoryInstance.Stub<IMember>();

            model.Id = 1234;
            model.FirstName = "first name";
            model.LastName = "last name";
            model.EmailAddress = "email address";
            model.ReceiveEmails = true;

            var serviceMock = new MembershipServiceMock();

            Assert.IsFalse(serviceMock.HasSaveBeenCalled, "Save method should not be called yet.");
            Assert.IsNull(serviceMock.ModelReceivedOnSaveCall, "The model property should be null.");

            var viewModel = new MemberEditorViewModel(model, serviceMock);

            viewModel.Id = 23456;
            viewModel.FirstName = "updated fn";
            viewModel.LastName = "updated ln";
            viewModel.EmailAddress = "updated email";
            viewModel.ReceiveEmails = !(viewModel.ReceiveEmails);

            viewModel.Save();

            Assert.IsTrue(serviceMock.HasSaveBeenCalled, "Save method should have been called.");
            Assert.IsNotNull(serviceMock.ModelReceivedOnSaveCall, "The model property should not be null.");
            Assert.AreSame(model, serviceMock.ModelReceivedOnSaveCall, "Wrong instance of the model.");

            AssertAreEqual(viewModel, model);
        }

        private void AssertAreEqual(MemberEditorViewModel expected, IMember actual)
        {
            Assert.IsNotNull(expected);
            Assert.IsNotNull(actual);

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.FirstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(expected.LastName, actual.LastName, "LastName");
            Assert.AreEqual<string>(expected.EmailAddress, actual.EmailAddress, "EmailAddress");
            Assert.AreEqual<bool>(expected.ReceiveEmails, actual.ReceiveEmails, "ReceiveEmails");
        }
    }
}
